//
//  UploadManager.m
//  AsReaderScan
//
//  Created by Y.Oshiro on 2021/09/01.
//

#import "UploadManager.h"
#import <TOSMBClient/TOSMBClient.h>
#import "FtpRequest.h"

@implementation UploadManager

static UploadManager *instance = nil;

#define Key_SMB_ServerURL @"Key_SMB_ServerURL"
#define Key_SMB_ServerName @"Key_SMB_ServerName"
#define Key_SMB_ServerPassword @"Key_SMB_ServerPassword"
#define Key_SMB_ServerFilePath @"Key_SMB_ServerFilePath"

#define Key_FTP_ServerURL @"Key_FTP_ServerURL"
#define Key_FTP_ServerName @"Key_FTP_ServerName"
#define Key_FTP_ServerPort @"Key_FTP_ServerPort"
#define Key_FTP_ServerPassword @"Key_FTP_ServerPassword"

+ (instancetype)sharedInstance
{
   @synchronized(self) {
       if (!instance) {
           instance = [UploadManager new];
       }
   }
   return instance;
}

#pragma mark - Private Method
//----------------------------------------------------------------------------------------
- (NSString *)filePath:(NSString *)fileName
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [paths objectAtIndex:0];
    NSString *filePath = [path stringByAppendingPathComponent:fileName];
    return filePath;
}

#pragma mark - Setter & Getter
//----------------------------------------------------------------------------------------

- (NSString *)serverURLSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_SMB_ServerURL];
}

-(void)setServerURLSMB:(NSString *)serverURLSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverURLSMB forKey:Key_SMB_ServerURL];
    [userDefaults synchronize];
}

- (NSString *)serverNameSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_SMB_ServerName];
}
- (void)setServerNameSMB:(NSString *)serverNameSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverNameSMB forKey:Key_SMB_ServerName];
    [userDefaults synchronize];
}
- (NSString *)serverPasswordSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_SMB_ServerPassword];
}
- (void)setServerPasswordSMB:(NSString *)serverPasswordSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverPasswordSMB forKey:Key_SMB_ServerPassword];
    [userDefaults synchronize];
}
- (NSString *)serverFilePathSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_SMB_ServerFilePath];
}
- (void)setServerFilePathSMB:(NSString *)serverFilePathSMB
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverFilePathSMB forKey:Key_SMB_ServerFilePath];
    [userDefaults synchronize];
}
- (NSString *)serverURLFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_FTP_ServerURL];
}
- (void)setServerURLFTP:(NSString *)serverURLFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverURLFTP forKey:Key_FTP_ServerURL];
    [userDefaults synchronize];
}
- (NSString *)serverNameFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_FTP_ServerName];
}
- (void)setServerNameFTP:(NSString *)serverNameFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverNameFTP forKey:Key_FTP_ServerName];
    [userDefaults synchronize];
}
- (NSString *)serverPortFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_FTP_ServerPort];
}
- (void)setServerPortFTP:(NSString *)serverPortFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverPortFTP forKey:Key_FTP_ServerPort];
    [userDefaults synchronize];
}
- (NSString *)serverPasswordFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [userDefaults objectForKey:Key_FTP_ServerPassword];
}
- (void)setServerPasswordFTP:(NSString *)serverPasswordFTP
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:serverPasswordFTP forKey:Key_FTP_ServerPassword];
    [userDefaults synchronize];
}

#pragma mark - Public Method
//----------------------------------------------------------------------------------------
// upload SMB
- (void)uploadSMBWithFileName:(NSString *)fileName complete:(void(^)(NSError *))complete
{
    NSString *severUrl = self.serverURLSMB;
    
    if ( severUrl && [severUrl hasPrefix:@"smb://"] ) {
        severUrl = [severUrl substringFromIndex:6];
    }
    TOSMBSession *session = [[TOSMBSession alloc]  initWithHostName:@"SMB" ipAddress:severUrl];
    [session setLoginCredentialsWithUserName:self.serverNameSMB password:self.serverPasswordSMB];
    NSString *filePath = [self filePath:fileName];
    NSData *data = [NSData dataWithContentsOfFile:filePath];
    NSString *uploadPath = [NSString stringWithFormat:@"/%@/%@", self.serverFilePathSMB, fileName];
    
    TOSMBSessionUploadTask *task = [session uploadTaskForFileAtPath:uploadPath data:data progressHandler:^(uint64_t totalBytesWritten, uint64_t totalBytesExpected) {
        //传输进度
        NSLog(@"SMB Client totalBytesWritten: %llu totalBytesExpected: %llu", totalBytesWritten,totalBytesExpected);
    } completionHandler:^{
        //传输结束
        NSLog(@"SMB Client Success");
        complete(nil);
        
    } failHandler:^(NSError *error) {
        //传输失败
        NSLog(@"SMB Client Error: %@", error.localizedDescription);
        complete(error);
    }];
    //开始传输
    [task resume];
}

// Upload FTP
- (void)uploadFTPWithFileName:(NSString *)fileName complete:(void(^)(NSError *error))complete
{
    FtpRequest *ftp = [[FtpRequest alloc] init];
    ftp.ftpPassword = self.serverPasswordFTP;
    ftp.ftpUser = self.serverNameFTP;
    ftp.localPath = [self filePath:fileName];
    ftp.serversPath = [NSString stringWithFormat:@"ftp://%@:%@/%@", self.serverURLFTP, self.serverPortFTP, fileName];
    
    [ftp upload:NO progress:^(Float32 percent, NSUInteger finishSize) {
        NSLog(@"-----%f,%lu",percent,(unsigned long)finishSize);
    } complete:^(id respond, NSError *error) {
        complete(error);
        NSLog(@"-----%@ , %@, %@, %@",respond, error,error.domain,error.userInfo);
    }];
}

@end
