//
//  AntiColViewController.m
//  Created by koda on 2015/02/20.
//
//

#import "AntiColViewController.h"
#import "ModelSettings.h"
#import "Utils.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface AntiColViewController ()<UITableViewDataSource, UITableViewDelegate, AsReaderRFIDDeviceDelegate, AsReaderDeviceDelegate>

@property (weak, nonatomic) IBOutlet UITableView *myTableView;
@property (strong, nonatomic) NSArray *modeList;
@property (assign, nonatomic) NSInteger selectedMode;
@property (assign, nonatomic) NSInteger selectedCounet;
@end

@implementation AntiColViewController

#define DEVICE_ASX_300R 302

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (void)viewDidLoad
{
    [super viewDidLoad];
    self.modeList = @[@"MODE 0", @"MODE 1", @"MODE 2", @"MODE 3", @"MODE 4", @"MODE 5", @"MODE 6"];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = self;
    device.delegateRFID = self;
    
    ModelSettings *model = [ModelSettings sharedInstance];
    _selectedMode = model.m_RFIDDeviceInfo.m_nAntiMode;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    AsReaderDevice *device = [AsReaderRFIDDevice sharedInstance];
    device.delegateDevice = nil;
    device.delegateRFID = nil;
}

#pragma mark - UITableView datasource
//----------------------------------------------------------------
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if( [[AsReaderInfo sharedInstance].deviceModelNumber isEqualToString:@"ASX-300R"]==YES ){
        return 7;
    } else {
        return 5;
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

#pragma mark - UITableView delegate
//----------------------------------------------------------------
- (UITableViewCell*)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    cell.textLabel.text = _modeList[indexPath.row];
    if (_selectedMode == indexPath.row) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    for (int i = 0; i < _modeList.count; i++) {
        UITableViewCell *cell = [_myTableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0]];
        if (i == indexPath.row) {
            cell.accessoryType = UITableViewCellAccessoryCheckmark;
            _selectedMode = indexPath.row;
        } else {
             cell.accessoryType = UITableViewCellAccessoryNone;
        }
    }
}

#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)done:(id)sender
{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        int nDefultCounter = 1;
        BOOL success = [[AsReaderRFIDDevice sharedInstance] setAnticollision:(int)self->_selectedMode Counter:nDefultCounter];
        if(success==NO ) {
            [Utils showAlert:@"" message:@"Error" view:self];
        }
    });
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)didSetAntiCollision:(int)status
{
    switch (status) {
        case 0x00: {
            dispatch_async(dispatch_get_main_queue(), ^{
                [Utils showAlert:@"" message:@"Anti-Collision Updated." view:self];
            });
        }
            break;
        default:
            break;
    }
}

- (void)plugged:(BOOL)plug
{
    
}

- (void)readerConnected:(int)status
{
    
}
@end
